#ifndef COMMUNICATION_H
#define COMMUNICATION_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
 
#include "thread.h"

#define ROSH_PORT 701701

void error(const char *msg)
{
    perror(msg);
    exit(0);
}

class Client{
    int sockfd;    
    public:
    
    int port;
    
    Client(char* host,int portno):port(portno){
        struct sockaddr_in serv_addr;
        struct hostent *server;
        
        sockfd = socket(AF_INET, SOCK_STREAM, 0);
        if (sockfd < 0) error("ERROR opening socket");
        
        server = gethostbyname(host);
        if(server == NULL) error("ERROR, no such host\n");
        
        bzero((char *) &serv_addr, sizeof(serv_addr));
        serv_addr.sin_family = AF_INET;
        bcopy((char *)server->h_addr,(char *)&serv_addr.sin_addr.s_addr,server->h_length);
        serv_addr.sin_port = htons(portno);
        if (connect(sockfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0) error("ERROR connecting");
    }
    
    int read(char* buff,long len){
        return ::recv(sockfd,buff,len,MSG_WAITALL);
        //return ::read(sockfd,buff,len);
    }
    
    int write(char* buff,long len){
        return ::write(sockfd,buff,len);
    }
    
    int send(char* msg){
        return ::write(sockfd,msg,strlen(msg));
    }
    
    ~Client(){
        close(sockfd);
    }
};


class Server{
    int connection;
    int sockfd;
    
    socklen_t clilen;
    sockaddr_in serv_addr, cli_addr;
        
    public:
    
    int port;
    
    Server(int portno,bool async = false):port(portno){
        clilen = sizeof(cli_addr);
        sockfd = socket(AF_INET, SOCK_STREAM, 0);
        if(sockfd < 0)error("ERROR opening socket");
        bzero((char *) &serv_addr, sizeof(serv_addr));

        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = INADDR_ANY;
        serv_addr.sin_port = htons(portno);
        if(bind(sockfd, (struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0) error("ERROR on binding");
    }
    
    void waitConnection(){
        listen(sockfd,5);
        connection = accept(sockfd, (struct sockaddr *) &cli_addr, &clilen);
    }
    
    int read(char* buff,long len){
        return ::read(connection,buff,len);
    }
    
    int write(char* buff,long len){
        return ::write(connection,buff,len);
    }
    
    /*int send(char* msg){
        return ::write(newsockfd,msg,strlen(msg));
    }*/
    
    void close(){
        ::close(sockfd);
        ::close(connection);
    }

};


/*class Communicator{
    public:
    Client master;
    
    Communicator():master("127.0.0.1",ROSH_PORT){
        
    }
    
    Server* newServer(char* name){
        int len = strlen(name);
        master.write((char*)&len,sizeof(int));
        master.write(name,sizeof(char)*len);
        int port;
        master.read((char*)&port,sizeof(int));
        return new Server(port,true);
    }
};*/
#endif