#ifndef LOCALIZATION_H
#define LOCALIZATION_H

#include "../Include/time.h"
#include "../Include/statistic.h"
#include "../Include/kinect.h"
#include "../Include/tree.h"
#include "../Include/point2D.h"
#include <iostream>

using namespace std;

/*void Dilate(Mat& binary,int dilate_size){
    dilate(binary,binary,getStructuringElement( MORPH_ELLIPSE,Size( 2*dilate_size + 1, 2*dilate_size+1 ),Point( dilate_size,dilate_size ) ));
}

class Map2D{
	public:

	kdTree<Point2D*,2>*tree;//map
    vector<Point2D*>points;//current points	
    float resolution;
    int dilation,sfuocation;
    Mat map;

	Map2D(kdTree<Point2D*,2>*t,float r,int d = 28,int s = 30):tree(t),resolution(r),dilation(d),sfuocation(s){
	 	float min_x = tree->minimize(0)->x;
	 	float min_y = tree->minimize(1)->y;
	 	float max_x = tree->maximize(0)->x;
	 	float max_y = tree->maximize(1)->y;	 	
	 	cout << min_x << " " << min_y << endl;
	 	cout << max_x << " " << max_y << endl;
	 	float delta_x = max_x - min_x;
	 	float delta_y = max_y - min_y;
	 	int res_x = ceil(delta_x/r);
	 	int res_y = ceil(delta_y/r);
	 	cout << res_x << " " << res_y << endl;
	 	map = Mat(res_y,res_x,CV_8UC1);
	 	vector<Point2D*>points = tree->vectorize();
	 	for(int a = 0;a < points.size();a++){
	 		int x = int((points[a]->x-min_x)/r+0.5);
	 		int y = int((points[a]->y-min_y)/r+0.5);
	 		map.at<uchar>(y,x) = 255;
	 	}
	 	imshow("Map",map);
	 	Mat occupancy = map.clone();
	 	Dilate(occupancy,d);
	 	blur(occupancy,occupancy,Size(s,s));
	 	equalizeHist(occupancy,occupancy);
	 	Dilate(occupancy,d/2);
	 	imshow("occupancy",occupancy);

	 	waitKey(0);
	}	
};*/

class kinectLineSlam{
	public:

	kdTree<Point2D*,2>*tree;//map
    vector<Point2D*>points;//current points

	int iterations;
	Transformation2D* t;
	float* smoothed_line;
	float alpha,momentum,range_limit;

	Point2D* point;

	kinectLineSlam(float a = 0.15,float m = 0.9,int i = 60,float r = 3.0):alpha(a),momentum(m),iterations(i),range_limit(r){
		t = new Transformation2D();
		smoothed_line = new float[640*3];
		point = new Point2D;
	}

	void getLine(float* line){
		//get the line and smooth it
	    //float* ptr = (float*)d.ptr(line);
		//smoothed_line = ptr;
		//smooth(ptr,smoothed_line,640,3);
		//add points to current mesure if not 0 and within range
	    points.clear();
	    for(int a = 0;a < 640;a++){
	        if(line[a*3+2] > 0.0 && line[a*3+2] < range_limit){
	        	points.push_back(new Point2D(line[a*3+0],line[a*3+2]));
        	}
	    }
	}

	void setReference(float* line){
		getLine(line);
		tree = new kdTree<Point2D*,2>(points);
	}

	void addMesure(float* line,Transformation2D* position){
    	getLine(line);
    	//set transformation to current estimated position
	    t->copy(position);
		Point2D *match,*actual;
		int size = points.size();
    	float rot_speed = 0;//momentum gradient descent for rotation

	    for(int a = 0;a < iterations;a++){
	        float rot_variation = 0;
	        float x_variation = 0;
	        float y_variation = 0;
        	//for every point (full batch)
	        for(int i = 0;i < size;i++){
	            actual = points[i];
	            t->transform(actual,point);
	            //get closest
	            match = tree->nearestNeighbor(point,0.01);
	            if(match == NULL)continue;
				//get x and y error on the transformed point
				float errX = match->x - point->x;
	            float errY = match->y - point->y;
	            //compute the gradient for the angle and the translations
	            float rot_der_x = -( actual->x*t->seno + actual->y*t->coseno);
	            float rot_der_y =  ( actual->x*t->coseno - actual->y*t->seno);
	            x_variation += errX*alpha;
	            y_variation += errY*alpha;
	            rot_variation += (rot_der_x*errX + rot_der_y*errY)*alpha;
			}
			//apply changes
			t->tx += x_variation/size;
	        t->ty += y_variation/size;
	        rot_speed = rot_speed*momentum + rot_variation/size;
	        t->angolo += rot_speed;
        	//compute sin and cos for current angle
	        t->trigo();
		}
	}

	void applyTransformation(Transformation2D* T,int subsample = 5){
		for(int a = 0;a < points.size();a+= subsample){
			T->transform(points[a]);
			tree->add(points[a]);
		}
	}

	void saveMap(char* name){
		cout << "Saving point map...";
		tree->save(name);
		cout << "OK\n";
	}
};

template<int dimensions,class Functor,class t = float>class ParticleFilter{
	public:

	struct particle{
		t data[dimensions];
		float w;
	};

	particle* actual;
	particle* next;
	vector<float*>actual_vector;
	int size;
	int best = 0;
	
	Functor* functor;//TODO explain

	float tot_w;

	ParticleFilter(int s,Functor* F,t* limits):size(s),functor(F){
		actual = new particle[s];
		for(int a = 0;a < s;a++){
			actual_vector.push_back(&actual[a].data[0]);
			for(int b = 0;b < dimensions;b++){
				actual[a].data[b] = randRange<t>(limits[b*2],limits[b*2+1]);	
			}
		}
		next = new particle[s];
	}

	void resample(int n = -1){ //n is the new size of the world (must be < size) 
		if(n < 0){//default n is the current size
			n = size;
		}
		//get current weights
		tot_w = 0;
		float m = 0;
		for(int a = 0;a < size;a++){
			actual[a].w = functor->getWeight(actual[a].data);
			if(actual[a].w > m){
				m = actual[a].w;
				best = a;
			}
			tot_w += actual[a].w;
		}
		//setup resampling wheel
		float step = tot_w/n;
		float start = randRange(0.0f,step);
		int i = 0;

		for(int a = 0;a < n;a++){//for each new particle
			while(actual[i].w < start){//keep moving until reaching step end
				start -= actual[i].w;
				i++;
				if(i >= size)i = 0;
			}
			//copy the particle
			next[a] = actual[i];
			//make the step
			start += step;
		}
		swap(next,actual);//swap next and actual     efficient because swap only pointers not the particles
		size = n;//update current world size
	}

	void propagate(){
		for(int a = 0;a < size;a++){
			functor->propagate(actual[a].data);
		}		
	}

	float* getBest(){
		return actual[best].data;
	}

};


class kinectLineLocalization{
	public:

	kdTree<Point2D*,2>tree;//map
    vector<Point2D*>points;//current points
    ParticleFilter<3,kinectLineLocalization>* particles;//3 dimensions (x,y,angle)

	int line,res;
	float motionError,senseError;
	Transformation2D* t;
	Space* plot;
	//float* smoothed_line;	

	kinectLineLocalization(Space* p,char* map,int particleNumber,float moveErr,float senseErr):plot(p),tree(map,loadPoint2D),motionError(moveErr),senseError(senseErr){
		vector<Point2D*>Points = tree.vectorize();
	    for(int a = 0;a < Points.size();a++)p->addPoint((float*)Points[a]);

    	float limits[6];
    	limits[0] = tree.minimize(0)->x-0.5;
    	limits[1] = tree.maximize(0)->x+0.5;
    	limits[2] = tree.minimize(1)->y-0.5;
    	limits[3] = tree.maximize(1)->y+0.5;
    	limits[4] = 0;
		limits[5] = PI*2;

	    particles = new ParticleFilter<3,kinectLineLocalization>(particleNumber,this,limits);
	    plot->red_points = particles->actual_vector;

		//this->smoothed_line = new float[640*3];
	
	}

	float* update(){
		particles->propagate();
		particles->resample();
		float* agent = particles->getBest();
		//cout << agent[0] << " " << agent[1] << endl;
 		plot->setAgent(agent[0],agent[1],agent[2]);
 		return agent;
	}

/*	void getLine(Mat& d){
		//get the line and smooth it
	    float* ptr = (float*)d.ptr(240);
		//smoothed_line = ptr;
		smooth(ptr,this->smoothed_line,640,3);
		//add points to current mesure if not 0 and within range
	    points.clear();
	    for(int a = 0;a < 640;a++){
	        if(smoothed_line[a*3+2] > 0.0 && smoothed_line[a*3+2] < 3.0){
	        	points.push_back(new Point2D(smoothed_line[a*3+0],-smoothed_line[a*3+2]));
        	}
	    }
	}*/

	void getLine(float* line){
		//get the line and smooth it
	    //float* ptr = (float*)d.ptr(line);
		//smoothed_line = ptr;
		//smooth(ptr,smoothed_line,640,3);
		//add points to current mesure if not 0 and within range
	    points.clear();
	    for(int a = 0;a < 640;a++){
	        if(line[a*3+2] > 0.0 && line[a*3+2] < 3.0){
	        	points.push_back(new Point2D(line[a*3+0],line[a*3+2]));
        	}
	    }
	}

	void propagate(float* point){
		point[0] += realisticError<float>(0,motionError);//randRange(-0.03,0.03);
		point[1] += realisticError<float>(0,motionError);//randRange(-0.03,0.03);
		point[2] += realisticError<float>(0,motionError);//randRange(-0.03,0.03);

	}

	float getWeight(float* point){
		Transformation2D t(point);
		Point2D* transformed = new Point2D;
		float distance = 0;
		int step = points.size()/20;
		for(int a = 0;a < points.size();a+=step){
			t.transform(points[a],transformed);
			Point2D* best = tree.nearestNeighbor(transformed,senseError);
			if(best == NULL){
				distance += senseError;
			}else{
				distance += tree.best_d;	
			}
		}
		return 1.0/distance;
	}
};


class TestLocalization{
	public:

	ParticleFilter<3,TestLocalization>* particles;//3 dimensions (x,y,angle)
	float center[2];

	Space* plot;

	TestLocalization(Space* p):plot(p){
    	float limits[6];
    	limits[0] = -1.5;
    	limits[1] = +1.5;
    	limits[2] = -1.5;
    	limits[3] = +1.5;
    	limits[4] = 0;
		limits[5] = PI*2;
		
		particles = new ParticleFilter<3,TestLocalization>(100,this,limits);
	    plot->red_points = particles->actual_vector;
	}

	void propagate(float* point){
		point[0] += randRange(-0.02,0.02);
		point[1] += randRange(-0.02,0.02);
	}

	void update(){
		center[0] += randRange(-0.2,0.2);
		center[1] += randRange(-0.2,0.2);
		particles->propagate();
		particles->resample();
	}

	float getWeight(float* point){
		return 1.0/(abs(point[0]-center[0])+abs(point[1]-center[1]));
	}	
};

#endif