#ifndef DATA_H
#define DATA_H

void patchZeros(float* ptr,int end,int max_len,float max_diff){
    for(int a = 1;a < end;a++){
        if(ptr[a] == 0){
            float prev = ptr[a-1];
            int b;
            for(b = 1;b < max_len;b++){
                if(ptr[a+b] > 0){
                    if(abs(ptr[a+b] - prev) < max_diff){
                        float diff = ptr[a+b]-prev;
                        float k = diff/b;
                        //float avg = (ptr[a+b] + prev)/2;
                        for(int c = 0;c < b;c++){
                            ptr[a+c] = prev + k*c;    
                        }
                    }
                    break;
                }
            }
        a = a+b;
        }
    }
}

#define SQRT_2_PI 2.506628

float gaussian(float mu,float sigma,float x){
    float res =  exp(    -pow(x-mu,2) / (2*pow(sigma,2))    );// / (sigma*SQRT_2_PI);
    //cout << mu << " " << sigma << " " << x << " " << res << endl;
    return res;
}

void smooth(float* src,float* res,int end,int smooth_len = 1){
    for(int a = 0;a < smooth_len;a++){
        res[a*3+0] = src[a*3+0];
        res[a*3+1] = src[a*3+1];
        res[a*3+2] = src[a*3+2];

    }
    for(int a = smooth_len;a < end-smooth_len;a++){
        res[a*3+0] = src[a*3+0];
        res[a*3+1] = src[a*3+1];
        int pos = a*3+2;
        res[pos] = src[pos];
        if(res[pos] > 0){
            float error = pow(res[pos],2)*0.0018;
            for(int b = -smooth_len;b < smooth_len;b++){
                res[pos] += (src[pos+b*3]-src[pos])*gaussian(src[pos],error,src[pos+b*3]);
            }
        }
    }
    for(int a = end-smooth_len;a < end;a++){
        res[a*3+0] = src[a*3+0];
        res[a*3+1] = src[a*3+1];
        res[a*3+2] = src[a*3+2];
    }
}

class Joystick{
	public:

	float x,y;
	SDL_Joystick* joy;
	float min,max;

	Joystick(float Min = 0.6,float Max = 0.99):joy(NULL),x(0),y(0),min(Min),max(Max){
		SDL_InitSubSystem(SDL_INIT_JOYSTICK);
		if(SDL_NumJoysticks() < 1){
			cout << CONSOLE_RED << "Error no joystick found\n" << CONSOLE_RESET;
		}else{
			joy = SDL_JoystickOpen ( 0 );
			SDL_JoystickEventState ( SDL_QUERY );
		}
	}

	void update(){
		SDL_JoystickUpdate();
		if(joy == NULL){
			cout << CONSOLE_RED << "Error no joystick\n" << CONSOLE_RESET;
		}else{
			float s = (32767.0-SDL_JoystickGetAxis(joy,2))/65534.0;
			//cout << s << endl;
			float speed = min+(max-min)*s;
            x = signedSquare(SDL_JoystickGetAxis(joy,0)/32767.3)*speed;
            y = signedSquare((-SDL_JoystickGetAxis(joy,1))/32767.3)*speed;
		}
	}
};

#endif
