#ifndef SERIAL_H
#define SERIAL_H

#include "console.h"

class Serial{
	public:

	int fd;

	Serial(const string name,int speed = 115200){
		fd = open(name.c_str(),O_RDWR | O_NOCTTY);
		if(fd == -1){
			cout << CONSOLE_RED << "Can't open serial port " << name << CONSOLE_RESET << endl;
			fail();
		}

		speed_t brate;

	    switch(speed) {
		    case 9600:   brate=B9600;   break;
		    case 19200:  brate=B19200;  break;
		    case 38400:  brate=B38400;  break;
		    case 57600:  brate=B57600;  break;
		    case 115200: brate=B115200; break;
		    default:
		    	cout << CONSOLE_RED << "Unknow baud rate " << speed << "(use 115200 instead)\n" << CONSOLE_RESET;
		    	brate=B115200;
		    	break;
	    }

		termios options;
		tcgetattr(fd, &options);
		cfsetispeed(&options, brate);
		cfsetospeed(&options, brate);
		
		 // 8N1
		options.c_cflag &= ~PARENB;
		options.c_cflag &= ~CSTOPB;
		options.c_cflag &= ~CSIZE;
		options.c_cflag |= CS8;
		// no flow control
		options.c_cflag &= ~CRTSCTS;

		//options.c_cflag &= ~HUPCL; // disable hang-up-on-close to avoid reset

		options.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
		options.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

		options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
		options.c_oflag &= ~OPOST; // make raw

		// see: http://unixwiz.net/techtips/termios-vmin-vtime.html
		options.c_cc[VMIN]  = 0;
		//options.c_cc[VTIME] = 0;
		options.c_cc[VTIME] = 1;

		tcsetattr(fd, TCSANOW, &options);
		if( tcsetattr(fd, TCSAFLUSH, &options) < 0) {
			cout << CONSOLE_RED << "Can't set attributes for serial port " << name << CONSOLE_RESET << endl;
			fail();
		}

	}

	template<class t>int  read(t& buf,size_t len = 1){
		return ::read(fd,(void*)&buf,len*sizeof(t));
	}

	template<class t>int write(t& buf,size_t len = 1){
		return ::write(fd,(void*)&buf,len*sizeof(t));
	}

	char getChar(){
		char res;
		this->read(res,1);
		return res;
	}

	void close(){
		::close(fd);
	}	
};

#endif