#include "../Include/roshbot.h"
#include <iostream>

using namespace std;

class MD49{
	public:

	Serial ser;
	char cmd[3];

	MD49(char * port):ser(port,38400){
		cmd[0] = 0x00;
		cmd[1] = 0x39; //enable timeout
		//0x38;//disable timeout
		ser.write(cmd,2);
		cmd[1] = 0x37;//enable pid
		ser.write(cmd,2);		
	}

	void setMode(char mode){
		cmd[1] = 0x34;
		cmd[2] = mode;
		ser.write(cmd,3);
	}

	int getVolts(){	
		cmd[1] = 0x26;
		ser.write(cmd,2);
		char res;
		ser.read(res,1);
		return res;
	}

	void setSpeed(uchar speed,char side){
		cmd[1] = (side == 0)?0x31:0x32;
		cmd[2] = speed;
		ser.write(cmd,3);
	}

	void resetEncoders(){
		cmd[1] = 0x35;
		ser.write(cmd,2);
	}

	int getEncoder(char n = 0){
		cmd[1] = (n == 0)?0x23:0x24;
		ser.write(cmd,2);
		int result;
		result =  ser.getChar() << 24ul;    // (0x00 shifted 24 bits left) 
		result += ser.getChar() << 16ul;    // (0x10 shifted 16 bits left) 
		result += ser.getChar() << 8ul;     // (0x56 shifted 8 bits left) 
		result += ser.getChar();            //  (0x32) 
		return result;
	}

	void getEncoders(int* enc){
		cmd[1] = 0x25;
		ser.write(cmd,2);
		for(int i = 0; i < 2; ++i){		
			enc[i] = ser.getChar() << 24;
			enc[i] |= (ser.getChar() << 16);
			enc[i] |= (ser.getChar() << 8);
			enc[i] |= (ser.getChar());		
		}        
	}

	float getAmps(char n = 0){
		cmd[1] = (n == 0)?0x27:0x28;
		ser.write(cmd,2);
		return ser.getChar()/10.0;
	}
};

class Odometry{
	public:

	double wheel_d;
	double x,y,phi;

	Odometry(double d):wheel_d(d),x(0),y(0),phi(0){
	}

	void update(double speed_s,double speed_d){
		//cout << speed_s << " " << speed_d << endl;
		if(speed_s == speed_d){
			x -= speed_s*sin(phi);
			y += speed_s*cos(phi);
		}else{
			double alpha = (speed_d-speed_s)/wheel_d;
			double r;

			if(speed_s < speed_d){
				r = -speed_s/tan(alpha) - wheel_d/2;
			}else{
				r = -speed_s/tan(alpha) - wheel_d/2;
			}
			//r = speed_s/tan(alpha);// - wheel_d/2;
			//if(speed_s < speed_d)r *= -1;
			//cout << alpha << " " << r << endl;

			double center_x = x + r*cos(phi);
			double center_y = y + r*sin(phi);
			//cout << center_x << " " << center_y << endl;

			double X = center_x + (x - center_x)*cos(alpha) -(y - center_y)*sin(alpha);
			y = center_y + (x - center_x)*sin(alpha) +(y - center_y)*cos(alpha);
			x = X;
			phi += alpha;

			//cout << pow(x-0.22,2)+y*y << endl;
		}
	}	
};
/*argv
	0: program name
input ports
	1: command
out ports
	
settings
	2: wheel distance
	3: steering powa
parent settings
	
*/
int main(int argc,char** argv){
	sharedMemoryInit();
	SharedMemClient joystick(atoi(argv[1]),sizeof(float)*2);
	float* joy = (float*)joystick.shm;
	
	MD49 motors("/dev/ttyUSB0");
	motors.setMode(1);
	motors.resetEncoders();
	int encoders [2];

	Odometry odo(float(atof(argv[2])));
	float meterPerTick = (0.125*PI)/980.0;

	Space space("Poligoni",800,800);
	Point2D* points = new Point2D[8];
	points[0] = Point2D(0.24,-0.065);
	points[1] = Point2D(-0.24,-0.065);
	points[2] = Point2D(-0.24,0.065);
	points[3] = Point2D(-0.20,0.065);
	points[4] = Point2D(-0.20,0.335);
	points[5] = Point2D(0.20,0.335);
	points[6] = Point2D(0.20,0.065);
	points[7] = Point2D(0.24,0.065);

	Poligon* poli = new Poligon(points,8); 
	space.poligons.push_back(poli);

	float centro[] = {0.22,0};
	space.points.push_back(centro);

	space.draw();
    
    while(true){
    	space.draw();

    	float steer = 1.0-abs(joy[0]);
    	char left = 127*(joy[0]-joy[1]*steer);
    	char right = 127*(joy[0]+joy[1]*steer);
    	//cout << int(left) << " " << int(right) << endl;
    	motors.setSpeed(left,0);
    	motors.setSpeed(right,1);

    	motors.getEncoders(encoders);
    	motors.resetEncoders();
    	float l = encoders[0]*meterPerTick;
    	float r = encoders[1]*meterPerTick;
    	cout << l << " " << r << endl;
    	odo.update(l,r);

    	//odo.update(0.001,0.003);

    	poli->t->angolo = odo.phi;
    	poli->t->tx = odo.x;
    	poli->t->ty = odo.y;
    	//poli->t->ty += 0.001;//realisticError(0.0,0.1);
    	poli->t->trigo();
    }
	return 0;
}