#!/usr/bin/env python


import os
import readline
import sys

def runcommand(cmd):
    print cmd
    os.system(cmd)

class Source:
    def __init__(self,line,master):
        parts = line.split(" ")
        self.master = master

        name = parts[0]
        
        self.pre = ""
        for a in range(1,len(parts)):
            self.addLib(parts[a].replace("\n",""))
        
        ext = name.split(".")[-1]
        if ext[0:2] == "cu":
            self.type = "cuda"
        elif ext[0:3] == "cpp":
            self.type = "c++"
        else:
            print "Cant detect type of "+ext
            self.type = ""
            
        self.name = ".".join(name.split(".")[0:-1])
        self.nameExt = name
        self.out = "outs/"+self.name+".o"
        
    def make(self):
        if self.type == "c++":
            self.gpp()
        elif self.type == "cuda":
            self.nvcc()
        else:
            print "fail"
    def makeFast(self):
        if self.type == "c++":
            self.gpp_fast()
        elif self.type == "cuda":
            self.nvcc_fast()
        else:
            print "fail"
    
    def gpp(self):
        print "Make "+self.name+".cpp"
        runcommand("g++ -g -c -std=c++11 "+self.pre+" "+self.name+".cpp -o  "+self.out+" > /dev/null")
        
    def gpp_fast(self):
        print "Make "+self.name+".cpp"
        runcommand("g++ -O3 -c -std=c++11 "+self.pre+" "+self.name+".cpp -o  "+self.out+" > /dev/null")
        
    def nvcc(self):
        print "Make "+self.name+".cu"
        runcommand("nvcc -g -c -arch=compute_30 "+self.name+".cu -o  "+self.out+" > /dev/null")
        
    def nvcc_fast(self):
        print "Make fast "+self.name+".cu"
        runcommand("nvcc -O3 -c -arch=compute_30 "+self.name+".cu -o  "+self.out+" > /dev/null")
        
    def addLib(self,lib):
        self.pre += os.popen("pkg-config "+lib+" --cflags").read()[:-1]
        if lib == "opencv":
            self.master.links += os.popen("pkg-config "+lib+" --libs").read()[:-1]
        else:
            self.master.links += os.popen("pkg-config "+lib+" --libs-only-L").read()[:-1]
            self.master.links += os.popen("pkg-config "+lib+" --libs-only-l").read()[:-1]

class Maker:
    def __init__(self,path):
        os.chdir(path)
        if not os.path.exists("outs"): os.makedirs("outs")
        f = open("makerfile")
        self.sources = list()
        self.links = ""
        for a in f: #for each line create a source file descriptor
            self.sources.append(Source(a,self))

    def link(self):
        res = "nvcc -g "
        for a in self.sources:
            res += a.out+" "
        print "Link"
        if self.links == "":
            runcommand(res+"-o main > /dev/null")
        else:
            runcommand(res+"-o main -L "+self.links+" > /dev/null")

    def make(self):
        for a in self.sources:
            a.make()
        self.link()

    def makeFast(self):
        for a in self.sources:
            a.makeFast()
        self.link()

    def run(self,args):
        os.system("./main "+args)

    def makeFile(self,name):
        for a in self.sources:
            if a.name == name:
                a.make()
                self.link()
                return 0
        print "No file called "+name
        for a in self.sources:
            print "    " + a.name

if __name__ == "__main__":
    print '\033[92m' + "Maker console" + "\033[0m"

    m = Maker(".")
    from console import *
    c = Console()
    c.addCommand("make","","make this package",lambda: m.make())
    c.addCommand("fast","","make this package with optimizations",lambda: m.makeFast())
    c.addCommand("run","run <args>","run this package",lambda *arg: m.run(" ".join(arg)))
    for a in m.sources:
        c.addCommand(a.name,"","make file "+a.nameExt+" and link",lambda: m.makeFile(a.name))
    #c.default = lambda args: m.makeFile(args[0])
    c.run()